#include <plib.h>
#include <p32xxxx.h>
#include "App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1 
// Disabld WDT , ICSP/ICD = PGC2 , PGD2 
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_8
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx2

#define SystemFrequency		60000000L
#define PBFrequency			SystemFrequency / 8
#define Timer1Tick			( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10

const unsigned char LCMString1[ ] = "PIC32MX795F512L "; 
const unsigned char LCMString2[ ] = "Write:  ,Read:  "; 
unsigned char Timer1TickFlag = 0;

#define EE_CS LATDbits.LATD12
#define EE_CS_TRIS TRISDbits.TRISD12

void SPI_EEByteWirte( unsigned int Addr , unsigned char Data );
unsigned char SPI_EEByteRead( unsigned int Addr );
void SPI_EEEnableWrite( void );

#define SPIChannel SPI_CHANNEL2	/* SPI_CHANNEL1 ~ SPI_CHANNEL4 */ 

void __ISR( _TIMER_1_VECTOR, ipl4 ) ISR_Imier1( void )
{
	Timer1TickFlag++;

	// MTimer_XСC
	mT1ClearIntFlag( );

	// LEDAϦVC	
	mPORTAToggleBits( BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 );
}

int main( void )
{
	unsigned long i = 0;
	unsigned int SPIEEAddr = 0;

	// disable JTAG port
    DDPCONbits.JTAGEN = 0;

	// Pɯ = tήɯ / 8C
	OSCSetPBDIV( OSC_PB_DIV_8 );

	// ϥC32SYSTEM ConfigNCPUާ@̨ΤơAקFPBDIV]wC
	SYSTEMConfig( SystemFrequency , SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE );
	// SYSTEMConfigPerformance( SystemFrequency  );
	// SYSTEMConfigWaitStates( SystemFrequency );

    // LEDI/O Port]wƦXAëwI/O PortAC
	mPORTASetPinsDigitalOut( BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_5 | BIT_6 | BIT_7 );
    mPORTASetBits( BIT_0 | BIT_1 | BIT_2 | BIT_3 );
	mPORTAClearBits( BIT_4 | BIT_5 | BIT_6 | BIT_7 );

	// ҥTimerAëw䤤_uvC
	ConfigIntTimer1( T1_INT_ON | T1_INT_PRIOR_4 );

	// Timer]wC
	OpenTimer1( T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT , Timer1Tick );

	// LCMlơC
	LCM_Init( );

	// ]wCСAXrC
	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	// To Do : ϥC32 SPI FunctionlSPI ModuleC
    // SPI Chip Select
    mPORTDSetPinsDigitalOut( BIT_12 );
	mPORTDSetBits( BIT_12 );

	// for v1.1.2
	//SpiChnConfigure( SPIChannel , SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV | SPI_OPEN_MODE8 | SPI_OPEN_FRZ );
	// for v2.0.1
	SpiChnConfigure( SPIChannel , SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV | SPI_OPEN_MODE8 | SPI_OPEN_SIDL );
	SpiChnSetBitRate( SPIChannel , PBFrequency , 100000 );
	SpiChnEnable( SPIChannel , 1 );

	// MPLAB C32 sª SPI s
		// New		// Old
		// SPI1		SPI1
		// SPI2		SPI2
		// SPI3		SPI1A
		// SPI4		SPI3A

	INTEnableSystemMultiVectoredInt( );

	while( 1 )
	{
		if( Timer1TickFlag >= 1 )
 		{
			Timer1TickFlag--;

			SPI_EEEnableWrite( );

			LCM_SetCursor( 6 , 1 );
			LCM_PutHex( SPIEEAddr );
			SPI_EEByteWirte( SPIEEAddr , SPIEEAddr ); 
		
			// ! EEPROMgJ, ݭn5mS, pGis, |_gJu@,
			// BH²檺n驵Bz, ѩ󤣦PsĶ,
			// n驵Ҳͪɶj@, SO`N!
			// Ϊ̧ĥεw骺pɨӳBz.
			for( i = 0; i < 50000L ; i++ );

			LCM_SetCursor( 14 , 1 );
			LCM_PutHex( SPI_EEByteRead( SPIEEAddr ) );

			if( ++SPIEEAddr > 0x7fff )
				SPIEEAddr = 0;
		}
	}

	while( 1 );
}

void SPI_EEByteWirte( unsigned int Addr , unsigned char Data )
{
	unsigned long i = 0;

	// CS Active
	EE_CS = 0;

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , 0x02 );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , ( Addr >> 8 ) & 0x00ff );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , Addr & 0xff );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , Data );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	EE_CS = 1;
	for( i = 0; i < 10 ; i++ );
}

unsigned char SPI_EEByteRead( unsigned int Addr )
{
	unsigned char Data = 0;
	unsigned long i = 0;

	// CS Active
	EE_CS = 0;

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , 0x03 );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , ( Addr >> 8 ) & 0x00ff );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , Addr & 0xff );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , Data );
	while( !SpiChnDataRdy( SPIChannel ) );
	Data = SpiChnReadC( SPIChannel );

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	EE_CS = 1;
	for( i = 0; i < 10 ; i++ );

	return Data;
}

void SPI_EEEnableWrite( void )
{
	unsigned long i = 0;

	// CS Active
	EE_CS = 0;

	while( SpiChnTxBuffFull( SPIChannel ) );
	SpiChnWriteC( SPIChannel , 0x06 );
	while( !SpiChnDataRdy( SPIChannel ) );
	SpiChnReadC( SPIChannel );						//Dummy Read

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	EE_CS = 1;
	for( i = 0; i < 10 ; i++ );
}
